'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __values(o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

var _a;
(function (CSSFetchState) {
    CSSFetchState[CSSFetchState["Loading"] = 0] = "Loading";
    CSSFetchState[CSSFetchState["Success"] = 1] = "Success";
    CSSFetchState[CSSFetchState["Error"] = 2] = "Error";
})(exports.CSSFetchState || (exports.CSSFetchState = {}));
/**
 * CSS Context
 */
var Provider = (_a = React.createContext({
    cssMap: new Map(),
    multiUpdate: function () { },
    update: function () { },
}), _a.Provider), Consumer = _a.Consumer;

/**
 * isServer
 * @return {boolean}
 */
function isServer() {
    return typeof window === 'undefined';
}

function createCSSCollector(renderLoading, renderError, displayName) {
    if (displayName === void 0) { displayName = 'CSSCollector'; }
    var ScopedCSSCollector = function (props) {
        return (React.createElement(Consumer, null, function (context) { return (React.createElement(CSSCollectorLifecycle, __assign({ context: context, renderError: renderError, renderLoading: renderLoading }, props))); }));
    };
    ScopedCSSCollector.displayName = displayName;
    return ScopedCSSCollector;
}
var CSSCollector = createCSSCollector();
var CSSCollectorLifecycle = /** @class */ (function (_super) {
    __extends(CSSCollectorLifecycle, _super);
    function CSSCollectorLifecycle(props) {
        var _this = _super.call(this, props) || this;
        var context = props.context, hrefs = props.hrefs;
        // When this instance created, access context api.
        context.multiUpdate(hrefs);
        return _this;
    }
    /**
     * get Fetch State from state array.
     * @param {CSSFetchState[]} states
     * @return {CSSFetchState}
     */
    CSSCollectorLifecycle.getFetchState = function (states) {
        var e_1, _a;
        var isLoading;
        try {
            for (var states_1 = __values(states), states_1_1 = states_1.next(); !states_1_1.done; states_1_1 = states_1.next()) {
                var state = states_1_1.value;
                // if some error state, return Error.
                if (state === exports.CSSFetchState.Error) {
                    return exports.CSSFetchState.Error;
                }
                if (state === exports.CSSFetchState.Loading || !state) {
                    isLoading = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (states_1_1 && !states_1_1.done && (_a = states_1.return)) _a.call(states_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (isLoading) {
            return exports.CSSFetchState.Loading;
        }
        else {
            return exports.CSSFetchState.Success;
        }
    };
    CSSCollectorLifecycle.prototype.render = function () {
        var _a = this.props, context = _a.context, hrefs = _a.hrefs;
        var state = CSSCollectorLifecycle.getFetchState(hrefs.map(function (href) {
            return context.cssMap.get(href);
        }));
        // if SSR, it should be render.
        if (state === exports.CSSFetchState.Success || isServer()) {
            return React.createElement(React.Fragment, null, this.props.children);
        }
        if (state === exports.CSSFetchState.Loading || !state) {
            return this.props.renderLoading ? this.props.renderLoading() : null;
        }
        if (state === exports.CSSFetchState.Error) {
            return this.props.renderError ? this.props.renderError() : null;
        }
        return null;
    };
    return CSSCollectorLifecycle;
}(React.Component));

/**
 * append css’s link element to head.
 * @param {string} href
 * @param {() => void} onLoad
 * @param {() => void} onError
 */
function appendCSS(href, onLoad, onError) {
    var linkElement = document.createElement('link');
    linkElement.href = href;
    linkElement.rel = 'stylesheet';
    linkElement.dataset.reactCssContext = 'true';
    linkElement.onload = function () {
        onLoad();
    };
    linkElement.onerror = function () {
        onError();
    };
    document.head.appendChild(linkElement);
}

var CSSProvider = /** @class */ (function (_super) {
    __extends(CSSProvider, _super);
    /**
     * @constructor
     * @param {Readonly<CSSProviderProps>} props
     */
    function CSSProvider(props) {
        var _this = _super.call(this, props) || this;
        /**
         * update CSS-Map in CSSContext.
         * @param {string} href
         */
        _this.update = function (href) {
            if (isServer()) {
                // if SSR
                // CSSCollector always pass the CSSLoadingState in SSR phase.
                // Then override the CSS-Map which is defined before server side rendering.
                _this.props.cssMap.set(href, exports.CSSFetchState.Success);
            }
            else {
                // if CSR
                // CSSCollector state is default Loading. (when css is not append yet.)
                // Append the link element with css.
                // When css file’s have been loaded, CSSCollector state will change to Success.
                // or if error, CSSCollector state will be Error.
                if (_this.state.cssMap.get(href) !== exports.CSSFetchState.Success) {
                    appendCSS(href, function () {
                        _this.updateCSSMap(href, exports.CSSFetchState.Success);
                    }, function () {
                        _this.updateCSSMap(href, exports.CSSFetchState.Error);
                    });
                }
            }
        };
        /**
         * multiple update CSS-Map in CSSContext.
         * @param {string[]} hrefs
         */
        _this.multiUpdate = function (hrefs) {
            hrefs.forEach(function (href) {
                _this.update(href);
            });
        };
        _this.state = {
            cssMap: props.cssMap,
            multiUpdate: _this.multiUpdate,
            update: _this.update,
        };
        return _this;
    }
    /**
     * set new CSS-Map state.
     * @param {string} href
     * @param {CSSFetchState} state
     */
    CSSProvider.prototype.updateCSSMap = function (href, state) {
        // create new CSS-Map to avoid override refs.
        var cssMap = new Map(this.state.cssMap.entries());
        cssMap.set(href, state);
        this.setState({
            cssMap: cssMap,
        });
    };
    CSSProvider.prototype.render = function () {
        return (React.createElement(Provider, { value: this.state }, React.Children.only(this.props.children)));
    };
    return CSSProvider;
}(React.Component));

/**
 * Get CSS-Map from head tag.
 * Can only use in browser.
 */
function getCSSMap() {
    var cssMap = new Map();
    var linkElements = document.querySelectorAll('link[data-react-css-context]');
    var nodeList = Array.prototype.slice.call(linkElements, 0);
    nodeList.forEach(function (link) {
        if (link && link.href) {
            cssMap.set(link.href, exports.CSSFetchState.Success);
        }
    });
    return cssMap;
}

/**
 * CSS-Map to HTMLLinkElement string.
 * @param {CSSMap} cssMap
 * @return {string}
 */
function toTagString(cssMap) {
    var e_1, _a;
    var tagString = '';
    try {
        for (var cssMap_1 = __values(cssMap), cssMap_1_1 = cssMap_1.next(); !cssMap_1_1.done; cssMap_1_1 = cssMap_1.next()) {
            var _b = __read(cssMap_1_1.value, 1), href = _b[0];
            if (href) {
                tagString += "<link href=\"" + href + "\" rel=\"stylesheet\" data-react-css-context=\"true\"/>";
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (cssMap_1_1 && !cssMap_1_1.done && (_a = cssMap_1.return)) _a.call(cssMap_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return tagString;
}
/**
 * CSS-Map to ReactElement.
 * @param {CSSMap} cssMap
 * @return {any}
 */
function toTagComponent(cssMap) {
    var e_2, _a;
    var hrefs = [];
    try {
        for (var cssMap_2 = __values(cssMap), cssMap_2_1 = cssMap_2.next(); !cssMap_2_1.done; cssMap_2_1 = cssMap_2.next()) {
            var _b = __read(cssMap_2_1.value, 1), href = _b[0];
            if (href) {
                hrefs.push(href);
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (cssMap_2_1 && !cssMap_2_1.done && (_a = cssMap_2.return)) _a.call(cssMap_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return (React.createElement(React.Fragment, null, hrefs.map(function (href) { return (React.createElement("link", { key: href, href: href, rel: "stylesheet", "data-react-css-context": true })); })));
}

exports.CSSCollector = CSSCollector;
exports.CSSProvider = CSSProvider;
exports.createCSSCollector = createCSSCollector;
exports.getCSSMap = getCSSMap;
exports.toTagComponent = toTagComponent;
exports.toTagString = toTagString;
